# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import messages as schema
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.volume import base_client


class MessagesClient(base_client.BaseClient):
    """Client class to send user messages API requests."""

    def show_message(self, message_id):
        """Show details for a single message."""
        url = 'messages/%s' % str(message_id)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_message, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_messages(self):
        """List all messages."""
        url = 'messages'
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_messages, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_message(self, message_id):
        """Delete a single message."""
        url = 'messages/%s' % str(message_id)
        resp, body = self.delete(url)
        self.validate_response(schema.delete_message, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_message(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'message'
